@@ -1,29 +1,27 @@
-
-/* ssdeep
-   (C) Copyright 2006 ManTech International Corporation
-
-   $Id: match.c 61 2008-02-22 23:18:59Z jessekornblum $
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-*/
+// ssdeep
+// (C) Copyright 2009 ManTech International Corporation
+//
+// $Id: match.c 80 2009-07-14 20:45:07Z jessekornblum $
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+// 
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+//You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 
 #include "ssdeep.h"
 
 
-/* The longest line we should encounter when reading files of known hashes */
+// The longest line we should encounter when reading files of known hashes 
 #define MAX_STR_LEN  2048
 
 
@@ -31,6 +29,9 @@
 
 int lsh_list_init(lsh_list *l)
 {
+  if (NULL == l)
+    return TRUE;
+
   l->top    = NULL;
   l->bottom = NULL;
   return FALSE;
@@ -39,6 +40,9 @@ int lsh_list_init(lsh_list *l)
 
 int match_init(state *s)
 {
+  if (NULL == s)
+    return TRUE;
+
   s->known_hashes = (lsh_list *)malloc(sizeof(lsh_list));
   if (s->known_hashes == NULL)
     return TRUE;
@@ -51,8 +55,14 @@ int match_init(state *s)
 
 #define STRINGS_EQUAL(A,B)    !_tcsncmp(A,B,MAX(_tcslen(A),_tcslen(B)))
 
-int match_compare(state *s, TCHAR *fn, char *sum)
+// Match the file named fn with the hash sum against the set of knowns
+// Display any matches. 
+// Return FALSE is there are no matches, TRUE if at least one match
+int match_compare(state *s, char * match_file, TCHAR *fn, char *sum)
 {
+  if (NULL == s || NULL == fn || NULL == sum)
+    fatal_error("%s: Null values passed into match_compare", __progname);
+
   size_t fn_len  = _tcslen(fn);
   size_t sum_len = strlen(sum);
 
@@ -64,12 +74,20 @@ int match_compare(state *s, TCHAR *fn, char *sum)
   {
     if (s->mode & mode_match_pretty)
     {
-      /* Prevent printing the redundant "A matches A" */
+      // Prevent printing the redundant "A matches A"
       if (!(_tcsncmp(fn,tmp->fn,MAX(fn_len,_tcslen(tmp->fn)))) &&
 	  !(strncmp(sum,tmp->hash,MAX(sum_len,strlen(tmp->hash)))))
       {
-	tmp = tmp->next;
-	continue;
+	// Unless these results from different matching files (such as
+	// what happens in sigcompare mode). That being said, we have to
+	// be careful to avoid NULL values such as when working in 
+	// normal pretty print mode.
+	if (NULL == match_file || NULL == tmp->match_file ||
+	    (!(strncmp(match_file, tmp->match_file, MAX(strlen(match_file),strlen(tmp->match_file))))))
+	{
+	  tmp = tmp->next;
+	  continue;
+	}
       }
     }
 
@@ -85,14 +103,18 @@ int match_compare(state *s, TCHAR *fn, char *sum)
 	}
       else
 	{
+	  if (match_file != NULL)
+	    printf ("%s:", match_file);
 	  display_filename(stdout,fn);
 	  printf(" matches ");
+	  if (tmp->match_file != NULL)
+	    printf ("%s:", tmp->match_file);
 	  display_filename(stdout,tmp->fn);
 	  print_status(" (%"PRIu32")", score);
 	}
       
-      /* We don't return right away as this file could match more than
-	 one signature.  */
+      // We don't return right away as this file could match more than
+      // one signature. 
       status = TRUE;
     }
     
@@ -103,10 +125,13 @@ int match_compare(state *s, TCHAR *fn, char *sum)
 }
 
 
-static int lsh_list_insert(state *s, lsh_list *l, TCHAR *fn, char *sum)
+static int lsh_list_insert(state *s, char * match_file, lsh_list *l, TCHAR *fn, char *sum)
 {
   lsh_node *new;
 
+  if (NULL == s || NULL == l || NULL == fn || NULL == sum)
+    return TRUE;
+
   if ((new = (lsh_node *)malloc(sizeof(lsh_node))) == NULL)
     fatal_error("%s: Out of memory", __progname);
 
@@ -117,6 +142,17 @@ static int lsh_list_insert(state *s, lsh_list *l, TCHAR *fn, char *sum)
     print_error(s,"%s: out of memory", __progname);
     return TRUE;
   }
+  if (match_file != NULL)
+  {
+    new->match_file = strdup(match_file);
+    if (NULL == new->match_file)
+    {
+      print_error(s,"%s: out of memory", __progname);
+      return TRUE;
+    }
+  }
+  else
+    new->match_file = NULL;
 
   if (l->bottom == NULL)
   {
@@ -136,11 +172,14 @@ static int lsh_list_insert(state *s, lsh_list *l, TCHAR *fn, char *sum)
 
 int match_pretty(state *s)
 {
+  if (NULL == s)
+    return TRUE;
+
   lsh_node *tmp = s->known_hashes->top;
 
   while (tmp != NULL)
   {
-    if (match_compare(s,tmp->fn,tmp->hash))
+    if (match_compare(s,tmp->match_file,tmp->fn,tmp->hash))
       print_status("");
 
     tmp = tmp->next;
@@ -150,9 +189,9 @@ int match_pretty(state *s)
 }
 
 
-int match_add(state *s, TCHAR *fn, char *hash)
+int match_add(state *s, char * match_file, TCHAR *fn, char *hash)
 {
-  return (lsh_list_insert(s,s->known_hashes,fn,hash));
+  return (lsh_list_insert(s,match_file,s->known_hashes,fn,hash));
 }
 
 
@@ -163,6 +202,9 @@ int match_load(state *s, char *fn)
   char *str, *known_hash;
   FILE *handle;
 
+  if (NULL == s || NULL == fn)
+    return TRUE;
+
   if ((handle = fopen(fn,"rb")) == NULL)
   {
     if (!(s->mode & mode_silent))
@@ -199,9 +241,8 @@ int match_load(state *s, char *fn)
   {
     chop_line(str);
 
-    /* The file format is:
-         hash,filename 
-    */
+    // The file format is:
+    //     hash,filename 
 
     strncpy(known_hash,str,MIN(MAX_STR_LEN,strlen(str)));
     find_comma_separated_string(known_hash,0);
@@ -220,7 +261,7 @@ int match_load(state *s, char *fn)
     }
     known_file_name[i] = 0;
     
-    if (match_add(s,known_file_name,known_hash))
+    if (match_add(s,fn,known_file_name,known_hash))
     {
       // If we can't insert this value, we're probably out of memory.
       // There's no sense trying to read the rest of the file.
